package com.hina.utils;

import com.alibaba.fastjson.JSON;
import com.hina.model.Event;
import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.bouncycastle.util.encoders.Hex;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.StandardCharsets;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.Security;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 数据加密工具
 */
public class DataEncryptUtil {

    private static final String BC = "BC";
    private static final String AES = "AES";
    private static final String AES_ECB_PKCS7PADDING = "AES/ECB/PKCS7Padding";

    /**
     * 加密密钥
     */
    private static final byte[] ENCRYPT_KEY = "1234567890123456".getBytes(StandardCharsets.UTF_8);

    static {
        // 只需要一次加载，不要重复new，可能会内存溢出
        if (Security.getProvider(BC) == null) {
            Security.addProvider(new BouncyCastleProvider());
        }
    }

    /**
     * 将原始json上报数据选取其中需要加密字段，对其值进行加密处理
     *
     * @param dataOriginalJson 原始报文json（报文格式，查看【2.2. 原始数据格式】）
     * @return 加密后的报文json格式（只允许对指定字段值加密，不允许对报文加密，报文中的json的key需要原值返回）
     */
    public static String encrypt(String dataOriginalJson) {
        List<Event> eventList = JSON.parseArray(dataOriginalJson, Event.class);
        for (Event event : eventList) {
            if (Objects.nonNull(event.getAnonymous_id())) {
                try {
                    byte[] cipherText = encryptAesEcb(event.getAnonymous_id(), ENCRYPT_KEY);
                    String anonymous_id = Hex.toHexString(cipherText);
                    event.setAnonymous_id(anonymous_id);
                } catch (Exception e) {
                    throw new RuntimeException(e);
                }
            }
        }
        return JSON.toJSONString(eventList);
    }

    /**
     * AES/ECB 加密（字符集：UTF8）
     *
     * @param plainText 明文
     * @param keyData   秘钥
     * @return 密文
     */
    private static byte[] encryptAesEcb(String plainText, byte[] keyData) throws NoSuchPaddingException, NoSuchAlgorithmException, InvalidKeyException, IllegalBlockSizeException, BadPaddingException {
        if (Objects.isNull(plainText)) {
            throw new RuntimeException("加密数据异常");
        }
        Integer[] keySize = {16, 24, 32};
        if (Objects.isNull(keyData) || !Arrays.stream(keySize).collect(Collectors.toList()).contains(keyData.length)) {
            throw new RuntimeException("加密秘钥异常");
        }
        SecretKeySpec key = new SecretKeySpec(keyData, AES);
        Cipher cipher = Cipher.getInstance(AES_ECB_PKCS7PADDING);//NOSONAR
        cipher.init(Cipher.ENCRYPT_MODE, key);
        return cipher.doFinal(plainText.getBytes(StandardCharsets.UTF_8));
    }

    public static void main(String[] args) {
        String eventJson = "[{\n" +
                "  \"anonymous_id\": \"C57174DA-4D5B-4BA7-A7B5-E740B3070C4F\",\n" +
                "  \"account_id\": \"10001\",\n" +
                "  \"event\": \"H_AppStart\",\n" +
                "  \"time\": 1704067200000,\n" +
                "  \"type\": \"track\",\n" +
                "  \"_track_id\": 123456,\n" +
                "  \"properties\": {\n" +
                "        \"自定义属性1\": 666,\n" +
                "        \"自定义属性2\": \"字符串\",\n" +
                "        \"自定义属性3\": \"2024-01-01 08:00:00\",\n" +
                "        \"预置属性H开头\": \"......\"\n" +
                "  }\n" +
                "}]";
        String result = encrypt(eventJson);
        System.out.println(result);
    }
}
